# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

__license__ = 'GNU Affero General Public License http://www.gnu.org/licenses/agpl.html'
__copyright__ = "Copyright (C) 2018 The OctoPrint Project - Released under terms of the AGPLv3 License"

import socket
import sys
import netaddr
import logging
import threading
import io
import os
import re

import netifaces
import requests

_cached_check_v6 = None
def check_v6():
	global _cached_check_v6

	def f():
		if not socket.has_ipv6:
			return False

		try:
			socket.socket(socket.AF_INET6, socket.SOCK_STREAM)
		except Exception:
			# "[Errno 97] Address family not supported by protocol" or anything else really...
			return False
		return True

	if _cached_check_v6 is None:
		_cached_check_v6 = f()
	return _cached_check_v6

HAS_V6 = check_v6()

if hasattr(socket, "IPPROTO_IPV6") and hasattr(socket, "IPV6_V6ONLY"):
	# Dual stack support, hooray!
	IPPROTO_IPV6 = socket.IPPROTO_IPV6
	IPV6_V6ONLY = socket.IPV6_V6ONLY
else:
	if sys.platform == "win32":
		# Python 2.7 on Windows lacks IPPROTO_IPV6, but supports the socket options just fine, let's redefine it
		IPPROTO_IPV6 = 41
		IPV6_V6ONLY = 27
	else:
		# Whatever we are running on here, we don't want to use V6 on here due to lack of dual stack support
		HAS_V6 = False

def is_lan_address(address, additional_private=None):
	if not address:
		# no address is LAN address
		return True

	try:
		address = unmap_v4_as_v6(address)
		address = strip_interface_tag(address)

		ip = netaddr.IPAddress(address)
		if ip.is_private() or ip.is_loopback():
			return True

		if additional_private is None or not isinstance(additional_private, (list, tuple)):
			additional_private = []

		subnets = netaddr.IPSet()
		for additional in additional_private:
			try:
				subnets.add(netaddr.IPNetwork(additional))
			except Exception:
				if logging.getLogger(__name__).isEnabledFor(logging.DEBUG):
					logging.getLogger(__name__).exception("Error while trying to add additional private network to local subnets: {}".format(additional))

		def to_ipnetwork(address):
			prefix = address["netmask"]
			if "/" in prefix:
				# v6 notation in netifaces output, e.g. "ffff:ffff:ffff:ffff::/64"
				_, prefix = prefix.split("/")

			addr = strip_interface_tag(address["addr"])
			return netaddr.IPNetwork("{}/{}".format(addr, prefix))

		for interface in netifaces.interfaces():
			addrs = netifaces.ifaddresses(interface)
			for v4 in addrs.get(socket.AF_INET, ()):
				try:
					subnets.add(to_ipnetwork(v4))
				except Exception:
					if logging.getLogger(__name__).isEnabledFor(logging.DEBUG):
						logging.getLogger(__name__).exception("Error while trying to add v4 network to local subnets: {!r}".format(v4))

			if HAS_V6:
				for v6 in addrs.get(socket.AF_INET6, ()):
					try:
						subnets.add(to_ipnetwork(v6))
					except Exception:
						if logging.getLogger(__name__).isEnabledFor(logging.DEBUG):
							logging.getLogger(__name__).exception("Error while trying to add v6 network to local subnets: {!r}".format(v6))

		if ip in subnets:
			return True

		return False

	except Exception:
		# we are extra careful here since an unhandled exception in this method will effectively nuke the whole UI
		logging.getLogger(__name__).exception("Error while trying to determine whether {} is a local address".format(address))
		return True


def strip_interface_tag(address):
	if "%" in address:
		# interface comment, e.g. "fe80::457f:bbee:d579:1063%wlan0"
		address = address[:address.find("%")]
	return address


def unmap_v4_as_v6(address):
	if address.lower().startswith("::ffff:") and "." in address:
		# ipv6 mapped ipv4 address, unmap
		address = address[len("::ffff:"):]
	return address


def interface_addresses(family=None):
	"""
	Retrieves all of the host's network interface addresses.
	"""

	import netifaces
	if not family:
		family = netifaces.AF_INET

	for interface in netifaces.interfaces():
		try:
			ifaddresses = netifaces.ifaddresses(interface)
		except Exception:
			continue
		if family in ifaddresses:
			for ifaddress in ifaddresses[family]:
				if not ifaddress["addr"].startswith("169.254."):
					yield ifaddress["addr"]


def address_for_client(host, port, timeout=3.05):
	"""
	Determines the address of the network interface on this host needed to connect to the indicated client host and port.
	"""

	for address in interface_addresses():
		try:
			if server_reachable(host, port, timeout=timeout, proto="udp", source=address):
				return address
		except Exception:
			continue


def server_reachable(host, port, timeout=3.05, proto="tcp", source=None):
	"""
	Checks if a server is reachable

	Args:
		host (str): host to check against
		port (int): port to check against
		timeout (float): timeout for check
		proto (str): ``tcp`` or ``udp``
		source (str): optional, socket used for check will be bound against this address if provided

	Returns:
		boolean: True if a connection to the server could be opened, False otherwise
	"""

	import socket

	if proto not in ("tcp", "udp"):
		raise ValueError("proto must be either 'tcp' or 'udp'")

	try:
		sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM if proto == "udp" else socket.SOCK_STREAM)
		sock.settimeout(timeout)
		if source is not None:
			sock.bind((source, 0))
		sock.connect((host, port))
		return True
	except Exception:
		return False

def resolve_host(host):
	import socket
	from octoprint.util import to_unicode

	try:
		return [to_unicode(x[4][0]) for x in socket.getaddrinfo(host, 80)]
	except Exception:
		return []


def download_file(url, folder, max_length=None):
	with requests.get(url, stream=True) as r:
		r.raise_for_status()
		if "Content-Disposition" in r.headers.keys():
			filename = re.findall("filename=(.+)", r.headers["Content-Disposition"])[0]
		else:
			filename = url.split("/")[-1]

		assert len(filename) > 0

		# TODO check content-length against safety limit

		path = os.path.join(folder, filename)
		with io.open(path, 'wb') as f:
			for chunk in r.iter_content(chunk_size=8192):
				f.write(chunk)
	return path



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

__author__ = "Gina Häußge <osd@foosel.net>"
__license__ = 'GNU Affero General Public License http://www.gnu.org/licenses/agpl.html'
__copyright__ = "Copyright (C) 2015 The OctoPrint Project - Released under terms of the AGPLv3 License"

import octoprint.plugin

class VirtualPrinterPlugin(octoprint.plugin.SettingsPlugin, octoprint.plugin.TemplatePlugin):

	def get_template_configs(self):
		return [
			dict(type="settings", custom_bindings=False)
		]

	def get_settings_defaults(self):
		return {
			"enabled": False,
			"okAfterResend": False,
			"forceChecksum": False,
			"numExtruders": 1,
			"pinnedExtruders": None,
			"includeCurrentToolInTemps": True,
			"includeFilenameInOpened": True,
			"hasBed": True,
			"hasChamber": False,
			"repetierStyleTargetTemperature": False,
			"okBeforeCommandOutput": False,
			"smoothieTemperatureReporting": False,
			"klipperTemperatureReporting": False,
			"reprapfwM114": False,
			"extendedSdFileList": False,
			"throttle": 0.01,
			"sendWait": True,
			"waitInterval": 1.0,
			"rxBuffer": 64,
			"commandBuffer": 4,
			"supportM112": True,
			"echoOnM117": True,
			"brokenM29": True,
			"brokenResend": False,
			"supportF": False,
			"firmwareName": "Virtual Marlin 1.0",
			"sharedNozzle": False,
			"sendBusy": False,
			"busyInterval": 2.0,
			"simulateReset": True,
			"resetLines": ['start', 'Marlin: Virtual Marlin!', '\x80', 'SD card ok'],
			"preparedOks": [],
			"okFormatString": "ok",
			"m115FormatString": "FIRMWARE_NAME:{firmware_name} PROTOCOL_VERSION:1.0",
			"m115ReportCapabilities": True,
			"capabilities": {
				"AUTOREPORT_TEMP": True,
				"AUTOREPORT_SD_STATUS": True,
				"EMERGENCY_PARSER": True
			},
			"m114FormatString": "X:{x} Y:{y} Z:{z} E:{e[current]} Count: A:{a} B:{b} C:{c}",
			"ambientTemperature": 21.3,
			"errors": {
				"checksum_mismatch": "Checksum mismatch",
				"checksum_missing": "Missing checksum",
				"lineno_mismatch": "expected line {} got {}",
				"lineno_missing": "No Line Number with checksum, Last Line: {}",
				"maxtemp": "MAXTEMP triggered!",
				"mintemp": "MINTEMP triggered!",
				"command_unknown": "Unknown command {}"
			}
		}

	def get_settings_version(self):
		return 1

	def on_settings_migrate(self, target, current):
		if current is None:
			config = self._settings.global_get(["devel", "virtualPrinter"])
			if config:
				self._logger.info("Migrating settings from devel.virtualPrinter to plugins.virtual_printer...")
				self._settings.global_set(["plugins", "virtual_printer"], config, force=True)
				self._settings.global_remove(["devel", "virtualPrinter"])

	def virtual_printer_factory(self, comm_instance, port, baudrate,
	                            read_timeout):
		if not port == "VIRTUAL":
			return None

		if not self._settings.get_boolean(["enabled"]):
			return None

		import logging.handlers
		from octoprint.logging.handlers import CleaningTimedRotatingFileHandler

		seriallog_handler = CleaningTimedRotatingFileHandler(self._settings.get_plugin_logfile_path(postfix="serial"),
		                                                     when="D",
		                                                     backupCount=3)
		seriallog_handler.setFormatter(logging.Formatter("%(asctime)s %(message)s"))
		seriallog_handler.setLevel(logging.DEBUG)

		from . import virtual

		serial_obj = virtual.VirtualPrinter(self._settings,
		                                    seriallog_handler=seriallog_handler,
		                                    read_timeout=float(read_timeout),
		                                    faked_baudrate=baudrate)
		return serial_obj

	def get_additional_port_names(self, *args, **kwargs):
		if self._settings.get_boolean(["enabled"]):
			return ["VIRTUAL"]
		else:
			return []

__plugin_name__ = "Virtual Printer"
__plugin_author__ = "Gina Häußge, based on work by Daid Braam"
__plugin_homepage__ = "https://docs.octoprint.org/en/master/development/virtual_printer.html"
__plugin_license__ = "AGPLv3"
__plugin_description__ = "Provides a virtual printer via a virtual serial port for development and testing purposes"
__plugin_pythoncompat__ = ">=2.7,<4"

def __plugin_load__():
	plugin = VirtualPrinterPlugin()

	global __plugin_implementation__
	__plugin_implementation__ = plugin

	global __plugin_hooks__
	__plugin_hooks__ = {
		"octoprint.comm.transport.serial.factory": plugin.virtual_printer_factory,
		"octoprint.comm.transport.serial.additional_port_names": plugin.get_additional_port_names
	}



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

__license__ = 'GNU Affero General Public License http://www.gnu.org/licenses/agpl.html'
__copyright__ = "Copyright (C) 2020 The OctoPrint Project - Released under terms of the AGPLv3 License"

import octoprint.plugin

from octoprint.access import USER_GROUP
from octoprint.access.permissions import Permissions
from octoprint.events import Events

import flask
from flask_babel import gettext

import time

class ActionCommandNotificationPlugin(octoprint.plugin.AssetPlugin,
                                      octoprint.plugin.SettingsPlugin,
                                      octoprint.plugin.SimpleApiPlugin,
                                      octoprint.plugin.TemplatePlugin,
                                      octoprint.plugin.EventHandlerPlugin):

	def __init__(self):
		self._notifications = []

	# Additional permissions hook

	def get_additional_permissions(self):
		return [
			dict(key="SHOW",
			     name="Show printer notifications",
			     description=gettext("Allows to see printer notifications"),
			     default_groups=[USER_GROUP],
			     roles=["show"]),
			dict(key="CLEAR",
			     name="Clear printer notifications",
			     description=gettext("Allows to clear printer notifications"),
			     default_groups=[USER_GROUP],
			     roles=["clear"])
		]

	#~ AssetPlugin

	def get_assets(self):
		return dict(js=["js/action_command_notification.js"],
		            clientjs=["clientjs/action_command_notification.js"],
		            css=["css/action_command_notification.css"])

	#~ EventHandlerPlugin

	def on_event(self, event, payload):
		if event == Events.DISCONNECTED:
			self._clear_notifications()

	#~ SettingsPlugin

	def get_settings_defaults(self):
		return dict(enable=True,
		            enable_popups=False)

	#~ SimpleApiPlugin

	def on_api_get(self, request):
		if not Permissions.PLUGIN_ACTION_COMMAND_NOTIFICATION_SHOW.can():
			return flask.abort(403, "Insufficient permissions")
		return flask.jsonify(notifications=[dict(timestamp=notification[0],
		                                         message=notification[1]) for notification in self._notifications])

	def get_api_commands(self):
		return dict(clear=[])

	def on_api_command(self, command, data):
		if command == "clear":
			if not Permissions.PLUGIN_ACTION_COMMAND_NOTIFICATION_CLEAR.can():
				return flask.abort(403, "Insufficient permissions")
			self._clear_notifications()

	#~ TemplatePlugin

	def get_template_configs(self):
		return [
			dict(type="settings",
			     name=gettext("Printer Notifications"),
			     custom_bindings=False),
			dict(type="sidebar",
			     name=gettext("Printer Notifications"),
			     icon="bell-o",
			     styles_wrapper=["display: none"],
			     data_bind="visible: loginState.hasPermissionKo(access.permissions.PLUGIN_ACTION_COMMAND_NOTIFICATION_SHOW)"
			               " && settings.settings.plugins.action_command_notification.enable()")
		]

	#~ action command handler

	def action_command_handler(self, comm, line, action, *args, **kwargs):
		if not self._settings.get_boolean(["enable"]):
			return

		parts = action.split(None, 1)
		if len(parts) == 1:
			action = parts[0]
			parameter = ""
		else:
			action, parameter = parts

		if action != "notification":
			return

		message = parameter.strip()
		self._notifications.append((time.time(), message))
		self._plugin_manager.send_plugin_message(self._identifier, dict(message=message))

		self._logger.info("Got a notification: {}".format(message))

	def _clear_notifications(self):
		self._notifications = []
		self._plugin_manager.send_plugin_message(self._identifier, dict())
		self._logger.info("Notifications cleared")

__plugin_name__ = "Action Command Notification Support"
__plugin_description__ = "Allows your printer to trigger notifications via action commands on the connection"
__plugin_author__ = "Gina Häußge"
__plugin_disabling_discouraged__ = gettext("Without this plugin your printer will no longer be able to trigger"
                                           " notifications in OctoPrint")
__plugin_license__ = "AGPLv3"
__plugin_pythoncompat__ = ">=2.7,<4"
__plugin_implementation__ = ActionCommandNotificationPlugin()
__plugin_hooks__ = {
	"octoprint.comm.protocol.action": __plugin_implementation__.action_command_handler,
	"octoprint.access.permissions": __plugin_implementation__.get_additional_permissions
}



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

__license__ = 'GNU Affero General Public License http://www.gnu.org/licenses/agpl.html'
__copyright__ = "Copyright (C) 2020 The OctoPrint Project - Released under terms of the AGPLv3 License"

import requests
import logging

from ..exceptions import ConfigurationInvalid, NetworkError, CannotCheckOffline

logger = logging.getLogger("octoprint.plugins.softwareupdate.version_checks.jsondata")

def get_latest(target, check, online=True):
	if not online:
		raise CannotCheckOffline()

	url = check.get("jsondata")
	current = check.get("current")

	if url is None:
		raise ConfigurationInvalid("jsondata version check needs jsondata set")

	try:
		with requests.get(url) as r:
			data = r.json()
	except Exception as exc:
		raise NetworkError(cause=exc)

	latest = data.get("version")

	information = dict(local=dict(name=current if current else "-", value=current),
	                   remote=dict(name=latest if latest else "-", value=latest))

	logger.debug("Target: {}, local: {}, remote: {}".format(target, information["local"]["name"], information["remote"]["name"]))

	return information, current is None or current == latest or latest is None



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

__license__ = 'GNU Affero General Public License http://www.gnu.org/licenses/agpl.html'
__copyright__ = "Copyright (C) 2020 The OctoPrint Project - Released under terms of the AGPLv3 License"


import threading
import logging

from octoprint.util.net import server_reachable, resolve_host

class ConnectivityChecker(object):
	"""
	Regularly checks for online connectivity.

	Tries to open a connection to the provided ``host`` and ``port`` every ``interval``
	seconds and sets the ``online`` status accordingly.

	If a ``name``is provided, also tries to resolve that name to a valid IP address
	during connectivity check and only set ``online`` to ``True`` if that succeeds as well.
	"""

	def __init__(self, interval, host, port, name=None, enabled=True, on_change=None):
		self._interval = interval
		self._host = host
		self._port = port
		self._name = name
		self._enabled = enabled
		self._on_change = on_change

		self._logger = logging.getLogger(__name__ + ".connectivity_checker")

		# we initialize the online flag to True if we are not enabled (we don't know any better
		# but these days it's probably a sane default)
		self._connection_working = not self._enabled
		self._resolution_working = not self._enabled or self._name is None

		self._check_worker = None
		self._check_mutex = threading.RLock()

		self._run()

	@property
	def online(self):
		"""Current online status, True if online, False if offline."""
		with self._check_mutex:
			return self._online

	@property
	def _online(self):
		return self._connection_working and self._resolution_working

	@property
	def host(self):
		"""DNS host to query."""
		with self._check_mutex:
			return self._host

	@host.setter
	def host(self, value):
		with self._check_mutex:
			self._host = value

	@property
	def port(self):
		"""DNS port to query."""
		with self._check_mutex:
			return self._port

	@port.setter
	def port(self, value):
		with self._check_mutex:
			self._port = value

	@property
	def name(self):
		with self._check_mutex:
			return self._name

	@name.setter
	def name(self, value):
		with self._check_mutex:
			self._name = value

	@property
	def interval(self):
		"""Interval between consecutive automatic checks."""
		return self._interval

	@interval.setter
	def interval(self, value):
		self._interval = value

	@property
	def enabled(self):
		"""Whether the check is enabled or not."""
		return self._enabled

	@enabled.setter
	def enabled(self, value):
		with self._check_mutex:
			old_enabled = self._enabled
			self._enabled = value

			if not self._enabled:
				if self._check_worker is not None:
					self._check_worker.cancel()

				old_value = self._online
				self._connection_working = self._resolution_working = True

				if old_value != self._online:
					self._trigger_change(old_value, self._online)

			elif self._enabled and not old_enabled:
				self._run()

	def check_immediately(self):
		"""Check immediately and return result."""
		with self._check_mutex:
			self._perform_check()
			return self.online

	def log_full_report(self):
		from octoprint.util import map_boolean

		with self._check_mutex:
			self._logger.info("Connectivity state is currently: {}".format(map_boolean(self.online, "online", "offline")))
			self.log_details()

	def log_change_report(self, old_value, new_value, include_details=False):
		from octoprint.util import map_boolean

		with self._check_mutex:
			self._logger.info("Connectivity changed from {} to {}".format(map_boolean(old_value, "online", "offline"),
			                                                              map_boolean(new_value, "online", "offline")))
			if include_details:
				self.log_details()

	def log_details(self):
		from octoprint.util import map_boolean
		self._logger.info("Connecting to {}:{} is {}".format(self._host,
		                                                     self._port,
		                                                     map_boolean(self._connection_working,
		                                                                 "working",
		                                                                 "not working")))
		if self._name:
			self._logger.info("Resolving {} is {}".format(self._name,
			                                              map_boolean(self._resolution_working,
			                                                          "working",
			                                                          "not working")))
	def _run(self):
		from octoprint.util import RepeatedTimer

		if not self._enabled:
			return

		if self._check_worker is not None:
			self._check_worker.cancel()

		self._check_worker = RepeatedTimer(self._interval, self._perform_check,
		                                   run_first=True)
		self._check_worker.start()

	def _perform_check(self):
		if not self._enabled:
			return

		with self._check_mutex:
			self._logger.debug("Checking against {}:{} if we are online...".format(self._host, self._port))

			old_value = self._online
			self._connection_working = server_reachable(self._host, port=self._port)

			if self._name:
				if self._connection_working:
					self._logger.debug("Checking if we can resolve {}...".format(self._name))
					self._resolution_working = len(resolve_host(self._name)) > 0
				else:
					self._resolution_working = False
			else:
				self._resolution_working = True

			if old_value != self._online:
				self._trigger_change(old_value, self._online)

	def _trigger_change(self, old_value, new_value):
		self.log_change_report(old_value, new_value, include_details=not new_value)
		if callable(self._on_change):
			self._on_change(old_value,
			                new_value,
			                connection_working=self._connection_working,
			                resolution_working=self._resolution_working)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

__license__ = 'GNU Affero General Public License http://www.gnu.org/licenses/agpl.html'
__copyright__ = "Copyright (C) 2020 The OctoPrint Project - Released under terms of the AGPLv3 License"

import logging
import shutil
import os
import ast
import io

from octoprint.util import TemporaryDirectory
from octoprint.util.net import download_file
from octoprint.settings import settings
from .. import exceptions

logger = logging.getLogger("octoprint.plugins.softwareupdate.updaters.download_single_file_plugin")

def can_perform_update(target, check, online=True):
	return online or check.get("offline", False)

def perform_update(target, check, target_version, log_cb=None, online=True, force=False):
	if not online and not check.get("offline", False):
		raise exceptions.CannotUpdateOffline()

	url = check.get("url")
	if url is None:
		raise exceptions.ConfigurationInvalid("download_single_file_plugin updater needs url set")

	def _log_call(*lines):
		_log(lines, prefix=" ", stream="call")

	def _log_message(*lines):
		_log(lines, prefix="#", stream="message")

	def _log(lines, prefix=None, stream=None):
		if log_cb is None:
			return
		log_cb(lines, prefix=prefix, stream=stream)

	folder = None
	try:
		try:
			_log_message("Download file from {}".format(url))
			folder = TemporaryDirectory()
			path = download_file(url, folder.name)
		except Exception as exc:
			raise exceptions.NetworkError(cause=exc)

		filename = os.path.basename(path)
		_, ext = os.path.splitext(filename)
		if ext not in (".py",):
			raise exceptions.UpdateError("File is not a python file: {}".format(filename), None)

		try:
			with io.open(path, 'rb') as f:
				ast.parse(f.read(), filename=path)
		except Exception:
			logger.exception("Could not parse {} as python file".format(path), None)
			raise exceptions.UpdateError("Could not parse {} as python file.".format(filename), None)

		destination = os.path.join(settings().getBaseFolder("plugins"), filename)

		try:
			_log_message("Copy {} to {}".format(path, destination))
			shutil.copy(path, destination)
		except Exception:
			logger.exception("Could not copy {} to {}".format(path, destination))
			raise exceptions.UpdateError("Could not copy {} to {}".format(path, destination), None)

		return "ok"
	finally:
		if folder is not None:
			folder.cleanup()



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

__license__ = 'GNU Affero General Public License http://www.gnu.org/licenses/agpl.html'
__copyright__ = "Copyright (C) 2020 The OctoPrint Project - Released under terms of the AGPLv3 License"

import octoprint.plugin

from flask_babel import gettext

class GcodeviewerPlugin(octoprint.plugin.AssetPlugin,
                        octoprint.plugin.TemplatePlugin,
                        octoprint.plugin.SettingsPlugin):

	def get_assets(self):
		js = ["js/gcodeviewer.js",
		      "js/viewer/ui.js",
		      "js/viewer/reader.js",
		      "js/viewer/renderer.js"
		]

		return dict(js=js,
		            less=["less/gcodeviewer.less"],
		            css=["css/gcodeviewer.css"])

	def get_template_configs(self):
		return [
			dict(type="tab",
			     template="gcodeviewer_tab.jinja2",
			     div="gcode",
			     styles=["display: none;"],
			     data_bind="visible: loginState.hasAllPermissionsKo(access.permissions.GCODE_VIEWER, access.permissions.FILES_DOWNLOAD)"),
			dict(type="settings",
			     template="gcodeviewer_settings.jinja2",
			     custom_bindings=True),
			dict(type="generic",
			     template="gcodeviewer_initscript.jinja2")
		]

	def get_settings_defaults(self):
		return dict(mobileSizeThreshold=2 * 1024 * 1024, # 2MB
		            sizeThreshold=20 * 1024 * 1024) # 20MB

	def get_settings_version(self):
		return 1

	def on_settings_migrate(self, target, current):
		if current is None:
			config = self._settings.global_get(["gcodeViewer"])
			if config:
				self._logger.info("Migrating settings from gcodeViewer to plugins.gcodeviewer...")
				if "mobileSizeThreshold" in config:
					self._settings.set_int(["mobileSizeThreshold"], config["mobileSizeThreshold"])
				if "sizeThreshold" in config:
					self._settings.set_int(["sizeThreshold"], config["sizeThreshold"])
				self._settings.global_remove(["gcodeViewer"])


__plugin_name__ = gettext("GCode Viewer")
__plugin_author__ = "Gina Häußge"
__plugin_description__ = "Provides a GCODE viewer in OctoPrint's UI."
__plugin_disabling_discouraged__ = gettext("Without this plugin the GCode Viewer in OctoPrint will no longer be "
                                           "available.")
__plugin_license__ = "AGPLv3"
__plugin_pythoncompat__ = ">=2.7,<4"
__plugin_implementation__ = GcodeviewerPlugin()
#__plugin_hooks__ = {
#	"octoprint.access.permissions": __plugin_implementation__.get_additional_permissions
#}
